%This function compares two imprinting strategies to see if a mutant can
%invade and replace the resident.  The answer to this can depend not only
%on the strategies, but also on the frequencies of the T and t alleles at
%mutation-selection balanace.  The frequencies of the T and t alleles at 
%mutation-selection balances can depend on the initial conditions.  

%If mating is random (no imprinting), then there is only one possible
%mutation-selection balance for the system.  The "bu" in this function's name
%indicates that we are studying invasion when the frequencies of T and t
%alleles in the population are those which would have evolved if imprinting
%had evolved by a series of small mutations.  We think this is the most
%biologically reasonable starting condition.

function [winner]=comp_bu(mir,soir,mii,soii,ct,cm,vf,vm,mu,f,m)
%'mir' and 'soir' are the mode and strength of imprinting for
%the resident phenotype, and 'mii' and 'soii' are the mode and strength of
%imprinting for the invader.


%As in find_msb, we will define some useful matrices up front
if mir=='p'
    imp_Pr=repmat([1,0,1,0],8,1);
    imp_pr=repmat([0,1,0,1],8,1);
end
if mir=='m'
    imp_Pr=repmat([1;1;0;0;1;1;0;0],1,4);
    imp_pr=repmat([0;0;1;1;0;0;1;1],1,4);
end

if mii=='p'
    imp_Pi=repmat([1,0,1,0],8,1);
    imp_pi=repmat([0,1,0,1],8,1);
end
if mii=='m'
    imp_Pi=repmat([1;1;0;0;1;1;0;0],1,4);
    imp_pi=repmat([0;0;1;1;0;0;1;1],1,4);
end

males_of_type_ST=[1,0.5,0.5,0.25;1,0.5,0.5,0.25;0.5,0,0.25,0;0.5,0,0.25,0;...
    0.5,0.25,0,0;0.5,0.25,0,0;0.25,0,0,0;0.25,0,0,0];
males_of_type_St=[0,0.5,0,0.25;0,0.5,0,0.25;0.5,1,0.25,0.5;0.5,1,0.25,0.5;...
    0,0.25,0,0;0,0.25,0,0;0.25,0.5,0,0;0.25,0.5,0,0];
males_of_type_sT=flipdim(flipdim(males_of_type_St,1),2);
males_of_type_st=flipdim(flipdim(males_of_type_ST,1),2);

%Additionally, we define acceptance rates as in find_msb
accr=1/exp(soir);
acci=1/exp(soii);

%Mating probabilities for each meeting
mating_prefs=[1,acci,1,acci;acci,1,acci,1;1,acci,1,acci;acci,1,acci,1;...
    1,accr,1,accr;accr,1,accr,1;1,accr,1,accr;accr,1,accr,1];
    
% %We set the initial population of the resident phenotype at 
% %mutation-selection balance
% [f,m]=find_msb(mir,soir,ct,cm,vf,vm,mu);

%Now, we introduce a small number of mutants with the new imprinting
%mode and strength.
f(1:4)=f(5:8)*.001;
f(5:8)=f(5:8)*.999;
m(1:2)=m(3:4)*.001;
m(3:4)=m(3:4)*.999;


muhold=[];

for gen=1:1000
    
    %Impose viability selection
    v_male=[1;exp(vm);1;exp(vm)];
    v_female=[1;1;exp(vf);exp(vf);1;1;exp(vf);exp(vf)];
    m=m.*v_male;
    f=f.*v_female;
    
    %Rescale populations to 1
    m=m/sum(m);
    f=f/sum(f);
    
    %This time we have to consider both fixed costs (which are higher
    %for the mutant) and relative costs.  Fixed costs are easy.  We
    %will assume that the fixed fitness cost of doubling choosiness is
    %constant (and thus choosiness increases exponentially with
    %investment).  We will assume that only females bear costs. So
    if ct=='f'
        f(1:4)=(1-cm*soii)*f(1:4);
        f(5:8)=(1-cm*soir)*f(5:8);
        f=f/sum(f);
    end 
    
    %If costs are relative, we can borrow the algorithm from find_msb,
    %modifying because the m and f vectors are now full.  Thus
    if ct=='r'
        f(1:4)=f(1:4).*[1-cm*m(4)*(1-acci)/(1-(1-cm)*m(4)*(1-acci));...
            1-cm*m(3)*(1-acci)/(1-(1-cm)*m(3)*(1-acci));...
            1-cm*m(4)*(1-acci)/(1-(1-cm)*m(4)*(1-acci));...
            1-cm*m(3)*(1-acci)/(1-(1-cm)*m(3)*(1-acci))];
        
        f(5:8)=f(5:8).*[1-cm*m(4)*(1-accr)/(1-(1-cm)*m(4)*(1-accr));...
            1-cm*m(3)*(1-accr)/(1-(1-cm)*m(3)*(1-accr));...
            1-cm*m(4)*(1-accr)/(1-(1-cm)*m(4)*(1-accr));...
            1-cm*m(3)*(1-accr)/(1-(1-cm)*m(3)*(1-accr))];
        
        f=f/sum(f);
    end
        
    %Now, we will do the matings as before.  We will borrow chuncks out of
    %the code we have already used.  The only thing we will do differently
    %is that we will now define our p-vectors according to the specific
    %mode of imprinting we are studying.  This will be useful later, since
    %it means we won't be constrained to have paternal imprinting in the
    %first four rows of the female population and maternal or oblique in
    %the second four rows.  (That will allow us to do things like compare a
    %paternal mode with one strength against a paternal mode with another.)
    %This is the reasoing behind creating the 'imp_P' matrices above.
    
    %Meeting frequency
    meetings=f*m';

    %Mating probabilities corrected for meeting frequency 
    mating_probs=meetings.*mating_prefs;
    
    %Scale the matrix so that each female mates exactly once
    mating_probs=mating_probs./repmat(sum(mating_probs,2),1,4);
    mating_probs(logical(isnan(mating_probs)))=0;
    mating_probs=mating_probs.*repmat(f,1,4);
   
    %These will be holders for the male and female populations while we
    %work on them
    o1=zeros(4,1);
    p=zeros(8,1);
    
    %Calculate males of each genotype resulting from matings
    o1(1)=sum(sum(mating_probs.*males_of_type_ST));
    o1(2)=sum(sum(mating_probs.*males_of_type_St));
    o1(3)=sum(sum(mating_probs.*males_of_type_sT));
    o1(4)=sum(sum(mating_probs.*males_of_type_st));
    
    %Calulate females of each phenogenotype in the next generation.  The
    %intial loop redifnes the 'imp_P' matrices if the mode of imprinting
    %is oblique
    if mii=='o'
        imp_Pi=m(1)+m(3);
        imp_pi=m(2)+m(4);
    end
    if  mir=='o'
        imp_Pr=m(1)+m(3);
        imp_pr=m(2)+m(4);
    end
    p(1)=sum(sum(mating_probs.*males_of_type_ST.*imp_Pi));
    p(2)=sum(sum(mating_probs.*males_of_type_ST.*imp_pi));
    p(3)=sum(sum(mating_probs.*males_of_type_St.*imp_Pi));
    p(4)=sum(sum(mating_probs.*males_of_type_St.*imp_pi));
    p(5)=sum(sum(mating_probs.*males_of_type_sT.*imp_Pr));
    p(6)=sum(sum(mating_probs.*males_of_type_sT.*imp_pr));
    p(7)=sum(sum(mating_probs.*males_of_type_st.*imp_Pr));
    p(8)=sum(sum(mating_probs.*males_of_type_st.*imp_pr));
   

    %Finally, we will allow mutations in the gene T, so that individuals
    %with genotype T change to t, and vice versa.  We will move the
    %densities back to m and f at the same time.
    m([1,3])=mu*o1([2,4])+(1-mu)*o1([1,3]);
    m([2,4])=(1-mu)*o1([2,4])+mu*o1([1,3]);
    f([1,2,5,6])=(1-mu)*p([1,2,5,6])+mu*p([3,4,7,8]);
    f([3,4,7,8])=mu*p([1,2,5,6])+(1-mu)*p([3,4,7,8]);
   
       
    if gen==500
        mutant500=(m(1)+m(2)+sum(f(1:4)))/2;
    end
    
    %muhold=[muhold;m',m2'];
    
end

%We ask whether the mutant wins by asking whether its density increases
%between generations 500 and 1000
if (m(1)+m(2)+sum(f(1:4)))/2<=mutant500 || mutant500==0
    winner=0;
else
    winner=1;
end



