%This function finds the mutation-selection balance of t in a population
%with a monomorphic imprinting strategy s.

function [f_at_msb,m_at_msb]=find_msb(mi,soi,ct,cm,vf,vm,mu)
%mi is the mode of imprinting, soi is the strength of imprinting, ct is the 
%cost type, cm is the cost magnitude, vf and vm are the effects of allele t 
%in males and females, and mu is the mutation rate.  

%First, we will define an imprinting matrix based on the mode of
%imprinting.  This is new, but the reason for it will become clear below.
if mi=='p'
    imp_P=repmat([1,0,1,0],8,1);
    imp_p=repmat([0,1,0,1],8,1);
end
if mi=='m'
    imp_P=repmat([1;1;0;0;1;1;0;0],1,4);
    imp_p=repmat([0;0;1;1;0;0;1;1],1,4);
end

%We can also define the Punnet square values that you already saw in
%Tramm.m
males_of_type_ST=[1,0.5,0.5,0.25;1,0.5,0.5,0.25;0.5,0,0.25,0;0.5,0,0.25,0;...
    0.5,0.25,0,0;0.5,0.25,0,0;0.25,0,0,0;0.25,0,0,0];
males_of_type_St=[0,0.5,0,0.25;0,0.5,0,0.25;0.5,1,0.25,0.5;0.5,1,0.25,0.5;...
    0,0.25,0,0;0,0.25,0,0;0.25,0.5,0,0;0.25,0.5,0,0];
males_of_type_sT=flipdim(flipdim(males_of_type_St,1),2);
males_of_type_st=flipdim(flipdim(males_of_type_ST,1),2);

%Our model is parameterized with soi, but in the dynamical equations
%below we will want to use the rate at which females accept mates of their
%less preferred phenotype instead.  So, lets define that here.

acc=1/exp(soi);
    
%We are going to track the dynamics of a population, and we need to have
%some starting point.  Where we start can matter.  For example, imagine
%that we start with a population that is fixed for a t gene with low
%viability, but in which females imprint strongly on their fathers.  When T
%alleles arise, they are more viable than t alleles.  However, T males are
%almost never chosen as mates.  So, sexual selection can keep the t
%allele common.  On the other hand, if we started with the T allele common,
%it would stay common. For now, let's start with the T and t alleles (almost)
%equally common.  In most cases, this should lead to the more viable allele
%becoming common, which is biologically reasonable.  (We may have to deal
%with the other case later, but let's not think about that for now.)  Note
%the perturbance in the third line below.  This is because the perfectly
%balanced state set by the first two lines is sometimes an unstable 
%equilibrium, and we want to avoid getting stuck there.
m=[0;0;.5;.5];
f=[0;0;0;0;.25;.25;.25;.25];
m=m.*(1+0.01*randn(4,1));
m=m/sum(m);

%Starting from here, we will iterate generations until we (almost) reach a
%mutation-selection balance.

%Unlike the for loop we used before, this while loop will stop only when
%the change in t is sufficiently small.  We can decide how small
%"sufficiently" is.  Here I have set it at 0.0001*mu.
delta_t=1;
old_t=0;
while delta_t>0.0001*mu
    
    %This checks to see if the change in t is small
    delta_t=abs(old_t-(m(4)+sum(f(7:8)))/2);
    old_t=(m(4)+sum(f(7:8)))/2;
    
    %Impose viability selection
    v_male=[1;exp(vm);1;exp(vm)];
    v_female=[1;1;exp(vf);exp(vf);1;1;exp(vf);exp(vf)];
    m=m.*v_male;
    f=f.*v_female;
    
    %Rescale populations to 1
    m=m/sum(m);
    f=f/sum(f);
    
    %Previously we had only had a cost associated with the t allele.  Now,
    %we will also introduce a cost of choosiness.  If the cost is fixed, we
    %don't have to worry about it here because everyone has the same s
    %allele.  If the cost is relative, we do need to worry about it here,
    %because the cost individuals incurr will be relative to the density of
    %their preferred mates.  In this case, we will assume that c is the
    %probability of dying between mating opportunities.  Males aren't
    %choosy, so they bear no cost.  For females, we can show:
    if ct=='r'
        f(5:8)=f(5:8).*[1-cm*m(4)*(1-acc)/(1-(1-cm)*m(4)*(1-acc));...
            1-cm*m(3)*(1-acc)/(1-(1-cm)*m(3)*(1-acc));...
            1-cm*m(4)*(1-acc)/(1-(1-cm)*m(4)*(1-acc));...
            1-cm*m(3)*(1-acc)/(1-(1-cm)*m(3)*(1-acc))];
        f=f/sum(f);
    end
        
    %Now, we will do the matings as before.  We will borrow chuncks out of
    %the code we have already used.  The only thing we will do differently
    %is that we will now define our p-vectors according to the specific
    %mode of imprinting we are studying.  This will be useful later, since
    %it means we won't be constrained to have paternal imprinting in the
    %first four rows of the female population and maternal or oblique in
    %the second four rows.  (That will allow us to do things like compare a
    %paternal mode with one strength against a paternal mode with another.)
    %This is the reasoing behind creating the 'imp_P' matrices above.
    
    %Meeting frequency
    meetings=f*m';
    
    %Mating probabilities for each meeting
    mating_prefs=[1,acc,1,acc;acc,1,acc,1;1,acc,1,acc;acc,1,acc,1;...
        1,acc,1,acc;acc,1,acc,1;1,acc,1,acc;acc,1,acc,1];

    %Mating probabilities corrected for meeting frequency 
    mating_probs=meetings.*mating_prefs;
    
    %Scale the matrix so that each female mates exactly once
    mating_probs=mating_probs./repmat(sum(mating_probs,2),1,4);
    mating_probs(logical(isnan(mating_probs)))=0;
    mating_probs=mating_probs.*repmat(f,1,4);
    
    %These will be holders for the male and female populations while we
    %work on them
    o=zeros(4,1);
    p=zeros(8,1);
    
    %Calculate males of each genotype resulting from matings
    o(1)=sum(sum(mating_probs.*males_of_type_ST));
    o(2)=sum(sum(mating_probs.*males_of_type_St));
    o(3)=sum(sum(mating_probs.*males_of_type_sT));
    o(4)=sum(sum(mating_probs.*males_of_type_st));
    
    %Calulate females of each phenogenotype in the next generation.  The
    %intial loop redifnes the 'imp_P' matrices if the mode of imprinting
    %is oblique
    if mi=='o'
        imp_P=m(1)+m(3);
        imp_p=m(2)+m(4);
    end
    p(1)=sum(sum(mating_probs.*males_of_type_ST.*imp_P));
    p(2)=sum(sum(mating_probs.*males_of_type_ST.*imp_p));
    p(3)=sum(sum(mating_probs.*males_of_type_St.*imp_P));
    p(4)=sum(sum(mating_probs.*males_of_type_St.*imp_p));
    p(5)=sum(sum(mating_probs.*males_of_type_sT.*imp_P));
    p(6)=sum(sum(mating_probs.*males_of_type_sT.*imp_p));
    p(7)=sum(sum(mating_probs.*males_of_type_st.*imp_P));
    p(8)=sum(sum(mating_probs.*males_of_type_st.*imp_p));

    %Finally, we will allow mutations in the gene T, so that individuals
    %with genotype T change to t, and vice versa.  We will move the
    %densities back to m and f at the same time.
    m([1,3])=mu*o([2,4])+(1-mu)*o([1,3]);
    m([2,4])=(1-mu)*o([2,4])+mu*o([1,3]);
    f([1,2,5,6])=(1-mu)*p([1,2,5,6])+mu*p([3,4,7,8]);
    f([3,4,7,8])=mu*p([1,2,5,6])+(1-mu)*p([3,4,7,8]);
    
end
    
f_at_msb=f;
m_at_msb=m;
